/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.workflow.api;

/**
 * Extended Workflow API
 * To use only in case of emergency
 *
 */
public interface IControlExtendedAPI {

	/**
	 * Checks if two nodes are on the same execution token, within the process with the specified process
	 * instance id
	 * @param processInstanceId Process instance id
	 * @param sourceTaskName Source task name
	 * @param destTaskName Destination task name
	 * @return True if the nodes are on the same execution token
	 */
	public boolean areNodesOnSameToken(long processInstanceId, String sourceTaskName, String destTaskName);

	/**
	 * Force the execution token on the node sourceTaskName, within the process identified by processInstanceId,
	 * to the task with name destTaskName.
	 * This is the method to use when the process instance has multiple tokens.
	 * @param processInstanceId Process instance id
	 * @param sourceTaskName Current task name
	 * @param destTaskName Destination task name
	 */
	public void forceToken(long processInstanceId, String sourceTaskName, String destTaskName);
	
	/**
	 * Force the unique execution token within the process identified by processInstanceId to
	 * the task with name destTaskName.
	 * @param processInstanceId Process instance id
	 * @param destTaskName Destination task name
	 */
	public void forceToken(long processInstanceId, String destTaskName);
	
}
