/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package mytest;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.StringTokenizer;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.eclipse.ebpm.workflow.api.IControlAPI;
import org.eclipse.ebpm.workflow.api.IProcessEngine;
import org.eclipse.ebpm.workflow.api.IQueryAPI;
import org.eclipse.ebpm.workflow.api.Process;
import org.eclipse.ebpm.workflow.api.ProcessInstance;
import org.eclipse.ebpm.workflow.api.Task;
import org.eclipse.ebpm.workflow.api.TaskInstance;
import org.eclipse.ebpm.workflow.api.Variable;
import org.slf4j.LoggerFactory;
import org.spagic.workflow.api.ws.client.WSProcessEngine;

public class APITester {
	
	protected IProcessEngine engine = null;
	protected IControlAPI controlAPI = null;
	protected IQueryAPI queryAPI = null;
	protected String actorId = null;
	
	
	// Id Process Instance, Process Name
	Map<Long, String> startedInstances = null;
	
	public APITester(){
		engine = new WSProcessEngine("http://localhost:9090/wfengine/");
		controlAPI = engine.getControlAPI();
		queryAPI = engine.getQueryAPI();
		
	}
	
	public void open() throws Exception {
		LoggerFactory.getLogger(this.getClass());
		try {
			String cmd = null;
		    Scanner in = new Scanner(System.in);

		    while (true){
		    	System.out.print(" -> "); 
		    	cmd = in.nextLine();
		    	
		    	if ((cmd.equalsIgnoreCase("stop")) || (cmd.equalsIgnoreCase("close")) || (cmd.equalsIgnoreCase("exit")))
		    		break;
		    	else
		    		try{
		    		
		    				execute(cmd);
		    			
		    		}catch (Throwable e) {
		    			e.printStackTrace();
		    		}
		    }
		
		} finally{
			engine.stop();
		}
	}
	
	public void execute(String executeCmd){
		StringTokenizer st  = new StringTokenizer(executeCmd);
		if (st.hasMoreTokens()){
			String cmd = st.nextToken().trim();
			if  (cmd.startsWith("help")){
				System.out.println(" ======================================= Command List ============================================ ");
				System.out.println(" def.list ");
				System.out.println(" def.tasks  !>");
				System.out.println(" new <definition_name!> ");
				System.out.println(" pi.list");
				System.out.println(" pi.list <definition_name?> ");
				System.out.println(" pi.abort <instanceID!> ");
				System.out.println(" pi.set <instanceID!> <varName!> <varValue!> ");
				System.out.println(" pi.state <instanceID!>  ");
				System.out.println(" ti <instancesId!>  ");		   							
				System.out.println(" ti.accept <instanceID!> <taskInstanceId!> ");      		
				System.out.println(" ti.complete <instanceID!> <taskInstanceId!>  ");     		
				System.out.println(" ti.reject <instanceID!> <taskInstanceId!> ");  
				if (actorId != null){	
					System.out.println(" my.tasklist  ( Accepted + Acceptable )");
					System.out.println(" my.department  ");
					System.out.println(" my.accepted ");
					System.out.println(" my.acceptable ");
					System.out.println(" my.valutatore");
					System.out.println("tasks.byprocess  <instanceID!>");
					System.out.println(" actor.unset");
				}
				else{
					System.out.println(" actor.set <actorId!> ");
				}
				System.out.println("stop"); 
			}else if (cmd.equalsIgnoreCase("def.list")){
				Process[] defs = queryAPI.getProcesses();
				showProcessList(defs);
			}else if (cmd.equalsIgnoreCase("def.tasks")){
				String processName = st.nextToken().trim();
				Task[] taskDefs = queryAPI.getHumanTask(processName);
				showTaskDefList(taskDefs);
			}else if  (cmd.equalsIgnoreCase("new")){
				String processName = st.nextToken().trim();
				long instanceId = controlAPI.startByProcessName(processName);
				System.out.println(instanceId + " Created ");
				
			}else if  (cmd.equalsIgnoreCase("pi.list")){
				String processName = null;
				ProcessInstance ins[] = null;
				if (st.hasMoreTokens()){
					processName = st.nextToken();
				}
				
				if (processName == null){
					ins = queryAPI.getProcessInstances();
				}else {
					ins = queryAPI.getProcessInstances(processName);
				}
				showProcessInstances(ins);
			}else if  (cmd.equalsIgnoreCase("pi.abort")){
				String instanceID = st.nextToken();
				controlAPI.abort(Long.valueOf(instanceID));
			}else if  (cmd.equalsIgnoreCase("pi.set")){
				if (st.countTokens() < 3){
					System.out.println(" The pi.set command must be in the form pi.set <INSTANCE_ID> <VAR_NAME> <VAR_VALUE>");
				}
				String instanceID = st.nextToken();
				String varName = st.nextToken();
				String varValue = st.nextToken();
				controlAPI.setGlobalVariable(Long.valueOf(instanceID), varName, varValue);
				
			}else if  (cmd.equalsIgnoreCase("pi.state")){
				if (st.countTokens() < 1){
					System.out.println(" The pi.state command must be in the form pi.state <INSTANCE_ID> ");
				}
				String instanceID = st.nextToken();
				Variable[] vars = controlAPI.getGlobalVariables(Long.valueOf(instanceID));
				
				showVariables(vars);
			}else if  (cmd.equalsIgnoreCase("ti")){
				if (st.countTokens() < 1){
					System.out.println(" The pi.state command must be in the form pi.state <INSTANCE_ID> ");
				}
				String instanceID = st.nextToken();
				TaskInstance[] tis = queryAPI.getHumanTaskInstances(Long.valueOf(instanceID));
				
				showTaskInstances(tis);
			}else if  (cmd.equalsIgnoreCase("ti.accept")){
				String instanceID = st.nextToken();
				String taskInstanceId = st.nextToken();
				
				if (actorId != null){
					controlAPI.startTaskByActor(actorId, Long.valueOf(instanceID), Long.valueOf(taskInstanceId), null);
				}else{
					controlAPI.startTask(Long.valueOf(instanceID), Long.valueOf(taskInstanceId), null);
				}
			}else if  (cmd.equalsIgnoreCase("ti.complete")){
				String instanceID = st.nextToken();
				String taskInstanceId = st.nextToken();
				
				if (actorId != null){
					controlAPI.executeTaskByActor(actorId, Long.valueOf(instanceID), Long.valueOf(taskInstanceId), null);
				}else{
					controlAPI.executeTask(Long.valueOf(instanceID), Long.valueOf(taskInstanceId), null);
				}
			}else if  (cmd.equalsIgnoreCase("ti.reject")){
				String instanceID = st.nextToken();
				String taskInstanceId = st.nextToken();
				
				if (actorId != null){
					controlAPI.rejectTaskByActor(actorId, Long.valueOf(instanceID), Long.valueOf(taskInstanceId));
				}else{
					controlAPI.rejectTask(Long.valueOf(instanceID), Long.valueOf(taskInstanceId));
				}
			}else if  (cmd.equalsIgnoreCase("my.tasklist")){
				
				
				TaskInstance[] tis = queryAPI.getHumanTaskInstancesForActor(actorId);
				showTaskInstances(tis);
				 
			
				//for ( int i = 0; i < 200; i ++) {
				//	System.out.println("i ["+i+"] Started");
				//	queryAPI.getGeneratedHumanTaskInstances();
				//	System.out.println("i ["+i+"] OK");
				
			
			}else if  (cmd.equalsIgnoreCase("my.department")){
				
				// torna i taskInstances in base al ruolo
				TaskInstance[] tis = queryAPI.getHumanTaskInstancesForActor(actorId);
				// funzione che filtra i task in base al dipertimento
				TaskInstance[] tdep = filtraDipartimento(tis);
				showTaskInstances(tdep);
			
			
			
			}else if  (cmd.equalsIgnoreCase("my.valutatore")){
			
				
				
				// check se l' utente si trova nella lista dei processi da valutare
				//implementazione lista in xml file
				
	
				List<TaskInstance> result = new ArrayList<TaskInstance>();
				
				//recuperare le istanze del processo
				List<Long> idProcessiVal = retrieveProcessEvaluate(actorId);
				
				//presentazione delle istanza dei task per cui l'utente  valutatore
				for (Long idProcess : idProcessiVal){
					TaskInstance[] tis = queryAPI.getHumanTaskInstancesForActorAndProcess(idProcess, "valutatore");
					if(tis != null && tis.length != 0){
						for(int i =0; i<tis.length ; i++)
							result.add(tis[i]);
					}
				}
				
				showTaskInstances(result.toArray(new TaskInstance[result.size()]));
			
			}else if(cmd.equalsIgnoreCase("tasks.byprocess")){
				// wrapper metodo di collezione dei task spettanti all'attore in base all' istanza di processo
				String instanceID = st.nextToken();
				Long processInstanceId = Long.parseLong(instanceID);
				
				//salvo l'attore attuale
				//String actor = actorId;
				
				//setto l'attore valuatore assegnandogli l'userId configurato nel sistema di profilatura
				//actorId = "valutatore";
				
				//prendo il task da assegnare al valutatore per la specifica istanza
				TaskInstance[] tis = queryAPI.getHumanTaskInstancesForActorAndProcess(processInstanceId, actorId);
				
				// risetto l'attore vero
				//actorId = actor;
				
				showTaskInstances(tis);
				
			}else if  (cmd.equalsIgnoreCase("my.accepted")){
				
				TaskInstance[] tis = queryAPI.getHumanTaskInstancesAssignedToActor(actorId);
				showAcceptedTaskInstances(tis);
			
			}else if  (cmd.equalsIgnoreCase("my.acceptable")){
				
				TaskInstance[] tis = queryAPI.getHumanTaskInstancesForActor(actorId);
				showAcceptableTaskInstances(tis);
			}else if  (cmd.equalsIgnoreCase("actor.unset")){
				this.actorId = null;
			}else if  (cmd.equalsIgnoreCase("actor.set")){
				this.actorId = st.nextToken().trim();
			}else{
				System.out.println("Comando non riconosciuto");
			}
		}
	}
	
	private List<Long> retrieveProcessEvaluate(String actorId2) {
		
		List<Long> result = new ArrayList<Long>();
		// accesso 
		SAXReader reader = new SAXReader();
		
		// salvataggio dell'istanza di processo in cui l'utente  valutatore
		try {
			Document doc = reader.read(new File("C:\\Scrappy\\valutatori.xml"));
			List<Node> nodes = doc.selectNodes("/VALUTATORI/*");
			for(Node n : nodes){
			 if(actorId2.equals(n.valueOf("@userID"))){
				Long l = Long.parseLong(n.valueOf("@processo"));
				 System.out.println("Utente :" + n.valueOf("@user")+" processo in cui  valutatore "+n.valueOf("@processo"));
				 result.add(l);
			 }
			 }} catch (DocumentException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
		
		return result;
	}

	private TaskInstance[] filtraDipartimento(TaskInstance[] tis) {
		//chiamata alla base dati di profilatura per estrapolare il dipartimento dello user
		String dep = retrieveDepartment(actorId);
		
		List<TaskInstance> result = new ArrayList<TaskInstance>(); 
		if(tis != null){
			for(TaskInstance t : tis){
				//retirve processInstanceId
				Long idp = t.getProcessInstanceId();
				//retrieve TaskInstace id
				//Long idt = t.getTaskInstanceId();
				//estrapolo il valore del dipartimento
				Variable v = controlAPI.getGlobalVariable(idp, "department");
				if(v.getValue().equals(dep))
					result.add(t);		
			}
		}
		return result.toArray(new TaskInstance[result.size()]);
	}

	private String retrieveDepartment(String actorId2) {
		// accesso 
		SAXReader reader = new SAXReader();
		String dep = null;
		try {
			Document doc = reader.read(new File("C:\\Scrappy\\userprofile.xml"));
			List<Node> nodes = doc.selectNodes("/AUTHORIZATIONS/ENTITIES/USERS/*");
			for(Node n : nodes){
			 if(actorId2.equals(n.valueOf("@userID"))){
				 dep = n.valueOf("@department");
				 System.out.println("Utente :" + n.valueOf("@user")+" dipartimento "+n.valueOf("@department"))	;
			 }
				 
			}
		
		} catch (DocumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
		return dep;
	}

	public String printStatus(int i){
		String status = "UNKNOW";
		if (i == 0){
			status = "NOT_GENERATED";
		}
		
		if (i == 1){
			status = "GENERATED";
		}
		
		
		if (i == 2){
			status =  "COMPLETED";
		}
		
		if (i == 3){
			status =  "CANCELLED";
		}
		return status;
	}
	
	public void showProcessList(org.eclipse.ebpm.workflow.api.Process[] defs){
		System.out.println("-- Process Definitions --");
		for (org.eclipse.ebpm.workflow.api.Process d : defs){
			System.out.println("-- Process ID ["+d.getProcessId()+"] Name ["+d.getName()+"] Version ["+d.getVersion()+"]");
		}
	}
	public void showTaskDefList(Task[] taskDefs){
		for ( Task task : taskDefs){
			System.out.println ( " -- Task ["+task.getTaskId()+"] ["+task.getTaskName()+"]");
			if (task.getEnabledRoles() != null){
				for ( String role : task.getEnabledRoles()){
					System.out.println ( " ----> Role ( "+ role + " )");	
				}
			}
		}
	}	
	public void showProcessInstances(ProcessInstance[] ins){
		for (ProcessInstance i : ins){
			System.out.println("-- Process Instance ["+i.getProcessInstanceId()+"] Name ["+i.getName()+"] State ["+i.getState()+"]");
		}
	}
	
	public void showVariables(Variable[] vars){
		for (Variable v : vars){
			System.out.println("-- Variable  [ "+v.getName()+"]");
			System.out.println(v.getValue());	
		}
	}
	
	public void showTaskInstances(TaskInstance[] tis){
		for (TaskInstance ti : tis){
			System.out.println("-- TI ["+ti.getTaskInstanceId()+"] - ["+ti.getProcessName() + "." + ti.getTaskName() +"] ("+printStatus(ti.getStatus())+") ( "+ti.getStatus()+")- OWNER ["+ti.getAssignedTo()+"]");
		}
	}
	
	public void showAcceptedTaskInstances(TaskInstance[] tis){	
		for (TaskInstance ti : tis){
			if ( ti.getAssignedTo() != null )
				System.out.println("-- TI ["+ti.getTaskInstanceId()+"] - ["+ti.getProcessName() + "." + ti.getTaskName() +"] ("+printStatus(ti.getStatus())+") - OWNER ["+ti.getAssignedTo()+"]");
		}
	}
	
	public void showAcceptableTaskInstances(TaskInstance[] tis){	
		for (TaskInstance ti : tis){
			if ( ti.getAssignedTo() == null )
				System.out.println("-- TI ["+ti.getTaskInstanceId()+"] - ["+ti.getProcessName() + "." + ti.getTaskName() +"] ("+printStatus(ti.getStatus())+") - OWNER ["+ti.getAssignedTo()+"]");
		}
	}
	
	public String printInstanceState(int i ){
		String state = "UNKNOW";
		
		if (i == 0)
			state = "RUNNING";
		
		if (i == 1)
			state = "EXECUTED";
		
		if (i == 2)
			state = "FAULTED";
		
		if (i == 3)
			state = "KILLED";
		
		if (i == 4)
			state = "EXPIRED";
		
		return state;
	}
	/*
	public void status(ProcessInstance pi){
		jbpmContext.save(pi);
		System.out.println(" ======================================================================================================");
		System.out.println("[ Process Instance ID ] -> ["+pi.getId()+"] of Process ["+pi.getProcessDefinition().getName()+"]");
		System.out.println("[ Process Instance Key ] -> ["+pi.getKey()+"]");
		System.out.println("[ Process Instance Version ] -> ["+pi.getVersion()+"]");
		System.out.println("[ Process Instance Started At] -> ["+pi.getStart()+"]" );
		System.out.println("[ Process Instance End At] -> ["+pi.getEnd()+"]" );
		System.out.println("[ Process Is Finished] -> ["+pi.hasEnded()+"]" );
		
		Token token = pi.getRootToken();
		System.out.println("[ Root Token ] -> ["+token.getNode().getName()+"]["+token.isTerminatedImplicitly()+"]");
		
		Collection<TaskInstance> tasks = getTasks(pi);
		System.out.println("[ Tasks ] -> ["+(tasks != null ? tasks.size() : 0)+"]");
		
		if (tasks != null){
			for (TaskInstance ti : tasks){
			
				System.out.println( "["+ti.getId()+"]["+ ti.getName()+"] -> ["+(ti.getEnd() != null ? "COMPLETED" : "STARTED")+"]["+ti.getActorId()+"]");
			}
		}
		System.out.println(" ======================================================================================================");
		
	}
	
	
	public Collection<TaskInstance> getTasks(ProcessInstance instance){
		return (Collection<TaskInstance>)instance.getTaskMgmtInstance().getTaskInstances();
		
	}
		*/
	public static void main(String[] args) {
		try{
			APITester tester = new APITester();
			tester.open();
		}catch (Exception e) {
			e.printStackTrace();
		}
	}

}
